const hx = require("hbuilderx")
const WorkspaceEdit = hx.WorkspaceEdit;
const TextEdit = hx.TextEdit;
const eslint = require("eslint");
const fs = require("fs");
const diff = require("./diff.js");
const baseConfig = require("./.eslintrc.js");
const pluginName = require("./package.json").name;

function resolveSettings(doc,autofix) {
	//TODO 这里应该用workspace.getWorkspaceFolder API
	let workspaceFolder = doc.workspaceFolder;

	let CLIEngine = eslint.CLIEngine;
	let cwd = process.cwd();
	if (workspaceFolder && workspaceFolder.uri) {
		let folderUri = workspaceFolder.uri;
		let userEslint = folderUri + "/node_modules/eslint";
		if (fs.existsSync(userEslint)) {
			CLIEngine = require(userEslint).CLIEngine;
			if (CLIEngine === undefined) {
				hx.window.showErrorMessage("当前项目下使用的eslint版本过低，请使用不低于1.0.0版本的eslint。");
				CLIEngine = eslint.CLIEngine;
			} else {
				cwd = folderUri;
			}
		}
	}
	return {
		library: {
			CLIEngine: CLIEngine
		},
		workingDirectory: cwd,
		options: {
			baseConfig: baseConfig,
			fix: autofix
		}
	}
}

function withCLIEngine(callback, settings) {
	const cwd = process.cwd();
	const newOptions = settings.options;
	try {
		if (settings.workingDirectory) {
			newOptions.cwd = settings.workingDirectory;
		}
		const cli = new settings.library.CLIEngine(newOptions);
		return callback(cli);
	} finally {
		if (cwd != process.cwd()) {
			process.chdir(cwd);
		}
	}
}

function validateAndFixDocument(doc,autofix) {
	if (doc.languageId !== "vue") {
		return;
	}
	hx.window.setStatusBarMessage("[" + pluginName + "]正在验证本文档语法...");
	let text = doc.getText();
	let settings = resolveSettings(doc,autofix);
	withCLIEngine(function(cli) {
		var report = cli.executeOnText(text, doc.fileName);
		let errorMsgs = [];
		if (report && report.results && Array.isArray(report.results) && report.results.length == 1) {
			let result = report.results[0];
			let messages = result.messages;
			if (messages && messages.length > 0) {
				for (let j = 0; j < messages.length; j++) {
					let errorMsg = messages[j];
					if (errorMsg.severity == 2) {
						errorMsgs.push({
							column: errorMsg.column - 1,
							line: errorMsg.line - 1,
							message: errorMsg.message
						});
					}
				}
			}
			let diagCollection = hx.languages.createDiagnosticCollection(pluginName);
			diagCollection.set(doc.uri, errorMsgs);
			//自动修复
			if (result.output !== undefined) {
				const diffs = diff.stringDiff(text, result.output, false);
				let workspaceEdit = new WorkspaceEdit();
				let edits = [];
				for (let df of diffs) {
					edits.push(new TextEdit({
						start: df.originalStart,
						end: df.originalStart + df.originalLength
					}, result.output.substr(df.modifiedStart, df.modifiedLength)));
				}
				workspaceEdit.set(doc.uri, edits);
				hx.workspace.applyEdit(workspaceEdit);
			}
		}
		let level = "info";
		let statusMsg = "[" + pluginName + "]验证完成，共发现" + errorMsgs.length + "个问题。";
		let showtime = 5*1000;
		if (errorMsgs.length > 0) {
			level = "error";
			statusMsg += "按[F4]可跳转到下一个错误。";
			showtime = 30*1000;
		}
		hx.window.setStatusBarMessage(statusMsg, showtime, level);
	}, settings)
}

function activate(context) {
	let autoFixCommandDispose = hx.commands.registerCommand("extension.eslint-vue.autoFix", function(context) {
		let editorPromise = hx.window.getActiveTextEditor();
		editorPromise.then((editor)=>{
			let doc = editor.document;
			validateAndFixDocument(doc,false);
		});
	});
    let eslintConfig = hx.workspace.getConfiguration("eslint-vue")
	//这个是执行保存格式化的时候用的
	let willSaveTextDocumentDispose = hx.workspace.onWillSaveTextDocument(function(event){
        let autoFixOnSave = eslintConfig.get("autoFixOnSave",true);
		validateAndFixDocument(event.document,autoFixOnSave);
    });
    let didChangeTextDocumentDispose = hx.workspace.onDidChangeTextDocument(function(event){
        let validateOnDocumentChanged = eslintConfig.get("validateOnDocumentChanged");
		if(validateOnDocumentChanged){
			validateAndFixDocument(event.document,false);
		}
    });
    
    context.subscriptions.push(autoFixCommandDispose);
    context.subscriptions.push(willSaveTextDocumentDispose);
    context.subscriptions.push(didChangeTextDocumentDispose);
}

module.exports = {
	activate: activate
}
